{***********************************************************************
Unit gfx_files.PAS v1.5  0801
    (c) by Andreas Moser, amoser@amoser.de,

    Delphi version : Delphi 4, 6 (due to user responses it seem to work with Delphi 5)


    gfx_files is part of the gfx_library collection

    You may use this sourcecode for your freewareproducts.
    You may modify this source-code for your own use.
    You may recompile this source-code for your own use.

    All functions, procedures and classes may NOT be used in commercial
    products without the permission of the author. For parts of this library
    not written by me, you have to ask for permission by their
    respective authors.

    Disclaimer of warranty: "This software is supplied as is. The author
    disclaims all warranties, expressed or implied, including, without
    limitation, the warranties of merchantability and of fitness for any
    purpose. The author assumes no liability for damages, direct or
    consequential, which may result from the use of this software."

    All brand and product names are marks or registered marks of their
    respective companies.

    Please report bugs to:
    Andreas Moser  amoser@amoser.de


********************************************************************************}

unit gfx_files;

{*****************************************

  N O T E :

  Because loss of time for coding fileconverters and the availability of
  much better packages, the gfx_lib file converters are only included for
  educational purposes. In fact, they are no longer a supported part of gfx_lib.

  I recommend the GraphicEx-Package from M.Lischke and/or the TGIFImage from A.Melander.

  If you would like to use Graphic-Ex, follow these steps:
  --------------------------------------------------------
  1.) Download these files
      GraphicEx.Zip from http://www.lieschke-online.de and
      CreateJPG.zip from http://www.lieschke-online.de
  2.) Unpack them to a directory of your choice (maybe you must add the paths to the
      Delphi environment library section)
  3.) remvove the "." from "$define USE_GRAPHIC_EX" after this instruction section


  If you would like to use TGifImage, follow these steps:
  --------------------------------------------------------
  1.) Download TGIFimage from Anders Melander (to be found on almost all Delphi Sites,
      e.g. http://www.torry.ru or Delphi Sunsite Mirrors
  2.) Install it as described in the package
  3.) remvove the "." from ".$define USE_TGIFIMAGE" after this instruction section
  4.) Make sure the "$define USE_GRAPHIC_EX" has a leading "."(".$define USE_GRAPHIC_EX")
      Maybe both packages will work being both active, but ive not tested it

  If you would like to use gfx_lib converters, follow these steps:
  --------------------------------------------------------
  1.) extract the files from gfx_converters.zip (included in gfxlibXX.zip)
  2.) remvove the "." from ".$define USE_GFX_LIB_CONVERTERS" after this instruction section
  3.) remvove the "." from ".$define USE_GFX_LIB_XXX" (for each filetype you want) after this instruction section
  4.) Make sure the "$define USE_GRAPHIC_EX" has a leading "."(".$define USE_GRAPHIC_EX")
  Additional information concerning PNG images: if you would like to access PNG images, you must download the
  TPNGImage component from Gustavo Daud. Unfortunately Gustavo has ceased the development of TPNGImage. But you can find the
  required units here: http://www.torry.net/vcl/graphics/other/thumbs.zip. Maybe you have to activate the "Procedure ReplaceTime(Var D:TDateTime; T:TDateTime);"
  in pngimage.pas.

  BUT NOTE: These converters wont be supported anymore, they are included just for educational purposes

*******************************************}


{.$define USE_GRAPHIC_EX}
{.$define USE_TGIFIMAGE}
{.$define USE_GFXLIB_CONVERTERS}

{.$define USE_GFXLIB_TIFF}
{.$define USE_GFXLIB_TGA}
{.$define USE_GFXLIB_PCD}
{.$define USE_GFXLIB_PCX}
{.$define USE_GFXLIB_PNG}





{$ifdef USE_GFXLIB_CONVERTERS}
  {$undef USE_GRAPHIC_EX}
{$endif}

{$ifdef USE_GRAPHIC_EX}
  {$undef USE_GFXLIB_CONVERTERS}
  {$undef USE_GFXLIB_TIFF}
  {$undef USE_GFXLIB_TGA}
  {$undef USE_GFXLIB_PCD}
  {$undef USE_GFXLIB_PCX}
  {$undef USE_GFXLIB_PNG}
  {$undef USE_TGIFIMAGE}
{$endif}


interface
uses JPEG,
     io_files,
     gfx_syswrapper,
     gfx_basedef,

     {$ifdef USE_GRAPHIC_EX}
        graphicEx,
     {$endif}

     {$ifdef USE_GFXLIB_TIFF}
        gfx_tiff,
     {$endif}

     {$ifdef USE_GFXLIB_TGA}
        gfx_tga,
     {$endif}

     {$ifdef USE_GFXLIB_PCD}
        gfx_pcd,
     {$endif}

     {$ifdef USE_GFXLIB_PCX}
        gfx_pcx,
     {$endif}

     {$ifdef USE_GFXLIB_PNG}
        pngimage,
     {$endif}

     {$ifdef USE_TGIFIMAGE}
        GIFImage,
     {$endif}

     gfx_errors;
{$R-}

//---------------------------------------------------------------------------
//
//			Global
//
// -----------------------------------------------------------------------------

//ModeA and ModeB are reserved for parameters passed to the converters

procedure LoadAnyImageToBMP(Bitmap:TXBitmap;Filename:String;AsThumb:Boolean;ModeA,ModeB:Integer);
procedure LoadJPEGExplicit(Graphic:TxGraphic;FileName:String;AsThumb:Boolean);
procedure SaveBMPToAnyFile(Bitmap:TXBitmap;Filename:String;ModeA,ModeB:Integer);
procedure SetJPEGParams(PixelFormat:TJPEGPixelFormat;Performance:TJPEGPerformance;
                          Scale:TJPEgScale;Grayscale:Boolean;Smoothing:Boolean;ProgressiveDispl:Boolean;SaveQuality:TJPEGQualityRange);
procedure SetJPEGDefaults;

// ** Internal procedures
procedure _LoadJPEG(Bitmap:TXBitmap;FileName:String;AsThumb:Boolean);
procedure _LoadBMP(Bitmap:TXBitmap;FileName:String);
procedure _LoadPCX(Bitmap:TXBitmap;FileName:String;ModeA,ModeB:Integer);
procedure _LoadTGA(Bitmap:TXBitmap;FileName:String;ModeA,ModeB:Integer);
procedure _LoadWMF(Bitmap:TXBitmap;FileName:String;ModeA,ModeB:Integer);
procedure _LoadPCD(Bitmap:TXBitmap;FileName:String;ModeA,ModeB:Integer);
procedure _LoadPNG(Bitmap:TXBitmap;FileName:String;ModeA,ModeB:Integer);
procedure _LoadGIF(Bitmap:TXBitmap;FileName:String;ModeA,ModeB:Integer);
procedure _SaveGIF(Bitmap:TXBitmap;FileName:String;ModeA,ModeB:Integer);
procedure _SaveJPEG(Bitmap:TXBitmap;FileName:String);
procedure _SaveBMP(Bitmap:TXBitmap;FileName:String);
procedure _LoadTIFF(Bitmap:TXBitmap;FileName:String;ModeA,ModeB:Integer);
procedure _SaveTIFF(Bitmap:TXBitmap;FileName:String;ModeA,ModeB:Integer);
procedure _SavePCX(Bitmap:TXBitmap;FileName:String;ModeA,ModeB:Integer);
procedure _SaveTGA(Bitmap:TXBitmap;FileName:String;ModeA,ModeB:Integer);
procedure _SavePNG(Bitmap:TXBitmap;FileName:String;ModeA,ModeB:Integer);

function GetProcessedWidth:Integer;
function GetProcessedHeight:Integer;
function GetProcessedCRC:Integer;
function GetProcessedDPI:Integer;
function GetProcessedPlanes:Integer;
function GetGenerateCRCState:Boolean;
function SetGenerateCRCState(State:Boolean):Boolean;

PROCEDURE PrintBitmap(Canvas:  TxCanvas; DestRect:  TxRect;  Bitmap:  TXBitmap);

Var _defJPEGPixelFormat:TJPEGPixelformat;
    _defJPEGPerformance:TJPEGPerformance;
    _defJPEGScale:      TJPEGScale;
    _defJPEGGrayScale:  Boolean;
    _defJPEGPrgrDisplay:Boolean;
    _defJPEGSmoothing:  Boolean;
    _defJPEGProgressEvent:TxProgressEvent;
    _defJPEGQualityRange:TJPEGQualityRange;
    _ProcessedImgWidth,
    _ProcessedImgHeight,
    _ProcessedImgDPI,
    _ProcessedImgPlanes:Integer;
    _GenerateCheckSums:Boolean;
    _ProcessedCRC32:xDWORD;

implementation





//***********************************************************
//
// Global
//
//***********************************************************


//---------------------------------------------------------------------------
//
//			LoadAnyImageToBMP
//
//-----------------------------------------------------------------------------
procedure LoadAnyImageToBMP(Bitmap:TXBitmap;Filename:String;AsThumb:Boolean;ModeA,ModeB:Integer);
var FileExt:String;
  {$ifdef USE_GRAPHIC_EX}
    Picture:TxPicture;
  {$endif}
begin

  _ProcessedImgWidth:=0;
  _ProcessedImgHeight:=0;
  _ProcessedCRC32:=0;
  if xFileExists(FileName) then
  begin
  FileExt:=xExtractFileExt(FileName);
  {$ifdef USE_GRAPHIC_EX}
          Picture:=TxPicture.Create;
          try
            Picture.LoadFromFile(FileName);
            if not (Picture.Graphic is TXBitmap) then
            begin
              with Bitmap do
              begin
                PixelFormat := xpf24Bit;
                Width := Picture.Width;
                Height := Picture.Height;
                Canvas.Draw(0, 0, Picture.Graphic);
               end;
             end else
             Bitmap.Assign(Picture.Graphic);

            finally
              Picture.Free;
            end;
  {$else}
    if FileExt='.bmp' then _LoadBMP(Bitmap,FileName);
    if FileExt='.jpg' then _LoadJPEG(Bitmap,FileName,AsThumb);
    {$ifdef USE_TGIFIMAGE}
      if FileExt='.gif' then _LoadGIF(Bitmap,FileName,ModeA,ModeB);
    {$endif}

    {$ifdef USE_GFXLIB_PCX}
      if FileExt='.pcx' then _LoadPCX(Bitmap,FileName,ModeA,ModeB);
    {$endif}

    {$ifdef USE_GFXLIB_TGA}
      if FileExt='.tga' then _LoadTGA(Bitmap,FileName,ModeA,ModeB);
    {$endif}

    {$ifdef USE_GFXLIB_PCD}
      if FileExt='.pcd' then _LoadPCD(Bitmap,FileName,ModeA,ModeB);
    {$endif}

    {$ifdef USE_GFXLIB_TIFF}
      if FileExt='.tif' then _LoadTIFF(Bitmap,FileName,ModeA,ModeB);
    {$endif}

    {$ifdef USE_GFXLIB_PNG}
      if FileExt='.png' then _LoadPNG(Bitmap,FileName,ModeA,ModeB);
    {$endif}

    if FileExt='.wmf' then _LoadWMF(Bitmap,FileName,ModeA,ModeB);
  {$endif}
  if _GenerateCheckSums then _ProcessedCRC32:=CRC32File(FileName);
  end;
end;

//---------------------------------------------------------------------------
//
//			SaveBMPToAnyFile
//
//-----------------------------------------------------------------------------
procedure SaveBMPToAnyFile(Bitmap:TXBitmap;Filename:String;ModeA,ModeB:Integer);
var FileExt:String;
begin
  FileExt:=xExtractFileExt(FileName);
  if FileExt='.bmp' then _SaveBMP(Bitmap,FileName) else
{$ifdef GIF}
  if FileExt='.gif' then _SaveGIF(Bitmap,FileName,ModeA,ModeB) else
{$endif}

{$ifdef USE_GFXLIB_PCX}
  if FileExt='.pcx' then _SavePCX(Bitmap,FileName,ModeA,ModeB) else
{$endif}

{$ifdef USE_GFXLIB_PNG}
  if FileExt='.png' then _SavePNG(Bitmap,FileName,ModeA,ModeB) else
{$endif}

{$ifdef USE_GFXLIB_TIFF}
  if FileExt='.tif' then _SaveTIFF(Bitmap,FileName,ModeA,ModeB) else
{$endif}

{$ifdef USE_GFXLIB_TGA}
  if FileExt='.tga' then _SaveTGA(Bitmap,FileName,ModeA,ModeB) else
{$endif}
  if (FileExt='.jpg') or (FileExt='.jpeg') then _SaveJPEG(Bitmap,FileName);
end;

//---------------------------------------------------------------------------
//
//			LoadJPEGExplicit
//
//-----------------------------------------------------------------------------
procedure LoadJPEGExplicit(Graphic:TxGraphic;FileName:String;AsThumb:Boolean);
var FileExt:String;
begin
  FileExt:=xExtractFileExt(FileName);
  if (FileExt='.jpg') or (FileExt='.jpeg') then
  begin
    Graphic.LoadFromFile(FileName);
  end;
end;

//---------------------------------------------------------------------------
//
//			SetJPEGParams
//
//-----------------------------------------------------------------------------
procedure SetJPEGParams(PixelFormat:TJPEGPixelFormat;Performance:TJPEGPerformance;
                          Scale:TJPEgScale;Grayscale:Boolean;Smoothing:Boolean;
                          ProgressiveDispl:Boolean;SaveQuality:TJPEGQualityRange);
begin
  _defJPEGSmoothing:=Smoothing;
  _defJPEGPrgrDisplay:=ProgressiveDispl;
  _defJPEGGrayScale:=Grayscale;
  _defJPEGScale:=Scale;
  _defJPEGPerformance:=Performance;
  _defJPEGPixelFormat:=PixelFormat;
  _defJPEGQualityRange:=SaveQuality;
end;

//---------------------------------------------------------------------------
//
//			SetJPEGDefaults
//
//---------------------------------------------------------------------------
procedure SetJPEGDefaults;
begin
 _defJPEGPixelFormat:=jf24Bit;
 _defJPEGPerformance:=jpBestQuality;
 _defJPEGScale      :=jsFullSize;
 _defJPEGGrayScale  :=False;
 _defJPEGPrgrDisplay:=False;
 _defJPEGSmoothing  :=True;
 _defJPEGProgressEvent:=nil;
 _defJPEGQualityRange:=90;
 _ProcessedImgWidth     :=0;
 _ProcessedImgHeight    :=0;
 _ProcessedImgDPI       :=0;
 _ProcessedImgPlanes    :=0;
end;

//---------------------------------------------------------------------------
//
//			_LoadJPEG
//
//---------------------------------------------------------------------------
procedure _LoadJPEG(Bitmap:TXBitmap;FileName:String;AsThumb:Boolean);
var TempJPEG:TJPEGImage;
begin
   TempJPEG:=TJPEGImage.Create;
  try
    if Assigned(_defJPEGProgressEvent) then TempJPEG.OnProgress:=_defJPEGProgressEvent;
    try
      TempJPEG.LoadFromFile(FileName);
      TempJPEG.PixelFormat:=jf24Bit;
      TempJPEG.Grayscale:=_defJPEGGrayScale;
      if (TempJPEG.Width>0) and (TempJPEG.Height>0) then
      with TempJPEG do
      begin
        _ProcessedImgWidth:=Width;
        _ProcessedImgHeight:=Height;
        Case PixelFormat of
          jf24Bit: _ProcessedImgPlanes:=24;
          jf8Bit: _ProcessedImgPlanes:=8;
        end;
        PixelFormat:=_defJPEGPixelFormat;
        ProgressiveDisplay:=_defJPEGPrgrDisplay;
        if AsThumb then
        begin
          if (Width/8)>70 then Scale:=jsEighth else
          if (Width/4)>70 then Scale:=jsQuarter else
          if (Width/2)>70 then Scale:=jsHalf;
        end
        else Scale:=_defJPEGScale;
        Smoothing:=_defJPEGSmoothing;
        Performance:=_defJPEGPerformance;
        Bitmap.Width:=TempJPEG.Width;
        Bitmap.Height:=TempJPEG.Height;
        case TempJPEG.PixelFormat of
          jf24Bit:begin
                    Bitmap.PixelFormat:=xpf24Bit;
                    Bitmap.Assign(TJPEGImage(TempJPEG));
                  end;
          jf8Bit: begin
                    TempJPEG.DIBNEeded;
                    Bitmap.PixelFormat:=xpf24Bit;
                    Bitmap.Assign(TJPEGImage(TempJPEG));
                  end;
        end;
      end else
      begin
        Bitmap.Width:=1;
        Bitmap.Height:=1;
      end;
    except
      Bitmap.Width:=1;
      Bitmap.Height:=1;
      TempJPEG.Free;
      if GFXRaiseErrors then raise  EGraphicFormat.Create('JPEG loading error');
      GFXFileErrorList.Add('JPEG loading error');
    end;
    finally
      TempJPEG.Free;
    end;

end;

//---------------------------------------------------------------------------
//
//			_LoadBMP
//
//---------------------------------------------------------------------------
procedure _LoadBMP(Bitmap:TXBitmap;FileName:String);
var tempBitmap:TXBitmap;
begin
  tempBitmap:=TXBitmap.Create;
  try
    tempBitmap.LoadFromFile(FileName);
    _ProcessedImgWidth:=tempBitmap.Width;
    _ProcessedImgHeight:=tempBitmap.Height;
    Bitmap.Assign(tempBitmap);
  finally
    tempBitmap.Free;
  end;
end;

//---------------------------------------------------------------------------
//
//			_LoadGIF
//
//---------------------------------------------------------------------------
procedure _LoadGIF(Bitmap:TXBitmap;FileName:String;ModeA,ModeB:Integer);
{$ifdef USE_TGIFIMAGE}
    var GifImage:TGifImage;
{$endif}
begin
{$ifdef USE_TGIFIMAGE}
        GIFImage := TGIFImage.Create;
        GifImage.Transparent:=False;
        TRY
          try
            GIFImage.LoadFromFile(Filename);
          except
                if GFXRaiseErrors then raise  EGraphicFormat.Create('GIF loading error');
          end;
          _ProcessedImgWidth:=Bitmap.Width;
          Bitmap.Assign(GifImage.Bitmap);
          Bitmap.PixelFormat := pf24bit;
          _ProcessedImgHeight:=Bitmap.Height;
        FINALLY
          GIFImage.Free
        end;
{$endif}
end;


//---------------------------------------------------------------------------
//
//			_LoadTIFF
//
//---------------------------------------------------------------------------
procedure _LoadTIFF(Bitmap:TXBitmap;FileName:String;ModeA,ModeB:Integer);
{$ifdef USE_GFXLIB_TIFF}
   var aTIFF:TTIFFFile;
{$endif}
begin
{$ifdef USE_GFXLIB_TIFF}
   aTIFF:=TTIFFFile.Create;
   aTIFF.LoadFromFile(FIleName);
   Bitmap.Assign(aTIFF.Bitmap);
   _ProcessedImgWidth:=Bitmap.Width;
   _ProcessedImgHeight:=Bitmap.Height;
   aTIFF.Free;
{$endif}
end;

procedure _LoadPCX(Bitmap:TXBitmap;FileName:String;ModeA,ModeB:Integer);
{$ifdef USE_GFXLIB_PCX}
    var aPcx:TPCXFile;
{$endif}
begin
{$ifdef USE_GFXLIB_PCX}
   aPcx:=TPCXFile.Create;
   aPcx.LoadFromFile(FIleName);
   Bitmap.Assign(aPcx.Bitmap);
   _ProcessedImgWidth:=Bitmap.Width;
   _ProcessedImgHeight:=Bitmap.Height;
   aPcx.Free;
{$endif}
end;

//---------------------------------------------------------------------------
//
//			_LoadPNG
//
//---------------------------------------------------------------------------
procedure _LoadPNG(Bitmap:TXBitmap;FileName:String;ModeA,ModeB:Integer);
{$ifdef USE_GFXLIB_PNG}
    var aPNG:TPNGImage;
{$endif}
begin
{$ifdef USE_GFXLIB_PNG}
   aPNG:=TPNGImage.Create;
   try
     aPNG.LoadFromFile(FIleName);
     Bitmap.Assign(aPNG);
     _ProcessedImgWidth:=Bitmap.Width;
     _ProcessedImgHeight:=Bitmap.Height;
   finally
     aPNG.Free;
   end;
{$endif}
end;

procedure _LoadTGA(Bitmap:TXBitmap;FileName:String;ModeA,ModeB:Integer);
{$ifdef USE_GFXLIB_TGA}
  var aTga:TTGAFile;
{$endif}
begin
{$ifdef USE_GFXLIB_TGA}
   atga:=TtgaFile.Create;
   atga.LoadFromFile(FIleName);
   Bitmap.Assign(atga.Bitmap);
   _ProcessedImgWidth:=Bitmap.Width;
   _ProcessedImgHeight:=Bitmap.Height;
   atga.Free;
{$endif}
end;

//---------------------------------------------------------------------------
//
//			_LoadWMF
//
//---------------------------------------------------------------------------
procedure _LoadWMF(Bitmap:TXBitmap;FileName:String;ModeA,ModeB:Integer);
var aWMF:TxMetaFile;
begin
   aWMF:=TxMetaFile.Create ;
   aWMF.LoadFromFile(FIleName);
   Bitmap.Width:=aWMF.Width;
   Bitmap.Height:=aWMF.Height;
   _ProcessedImgWidth:=Bitmap.Width;
   _ProcessedImgHeight:=Bitmap.Height;
   Bitmap.Canvas.Draw(0,0,aWMF);
   aWMF.Free;
end;

//---------------------------------------------------------------------------
//
//			_LoadPCD
//
//---------------------------------------------------------------------------
procedure _LoadPCD(Bitmap:TXBitmap;FileName:String;ModeA,ModeB:Integer);
{$ifdef USE_GFXLIB_PCD}
   var aPCD:TPCDFile;
{$endif}
begin
{$ifdef USE_GFXLIB_PCD}
   aPCD:=TPCDFile.Create;
   if ModeA=0 then ModeA:=3;
   PCDSize:=ModeA;
   aPCD.LoadFromFile(FIleName);
   Bitmap.Assign(aPCD.Bitmap);
   _ProcessedImgWidth:=Bitmap.Width;
   _ProcessedImgHeight:=Bitmap.Height;
   aPCD.Free;
{$endif}
end;

//---------------------------------------------------------------------------
//
//			_SaveBMP
//
//---------------------------------------------------------------------------
procedure _SaveBMP(Bitmap:TXBitmap;FileName:String);
begin
    Bitmap.SaveToFile(FileName);
end;

//---------------------------------------------------------------------------
//
//			_SaveJPEG
//
//---------------------------------------------------------------------------
procedure _SaveJPEG(Bitmap:TXBitmap;FileName:String);
Var TempJPEG:TJPegImage;
begin
   TempJPEG:=TJPEGImage.Create;
   try
     TempJPEG.Assign(Bitmap);
     TempJPEG.CompressionQuality:=_defJPEGQualityRange;
     TempJPEG.SaveToFile(FileName);
   finally
     TempJPEG.Free;
   end;
end;

//---------------------------------------------------------------------------
//
//			_SaveGIF
//
//---------------------------------------------------------------------------
procedure _SaveGIF(Bitmap:TXBitmap;FileName:String;ModeA,ModeB:Integer);

{$ifdef USE_TGIFIMAGE}
  var GifImage:TGifImage;
{$endif}

begin
{$ifdef USE_TGIFIMAGE}
        Bitmap := ReduceColors(Bitmap, rmQuantize,dmNearest, 8,Bitmap.palette);// from Gifimage, reduce the bitmap palette  with the nearest colors.
        GIFImage := TGIFImage.Create;
        GifImage.Transparent:=False;
        TRY
          GIFImage.Height:=Bitmap.Height;
          GIFImage.Width:=Bitmap.Width;
          GifImage.Compression:=gcRLE; // no royalties for this GIF format
          GifImage.Assign(Bitmap);
          GifImage.SaveToFile(FileName);
        FINALLY
          GIFImage.Free
        end;
{$endif}
end;



//---------------------------------------------------------------------------
//
//			_SaveTIFF
//
//---------------------------------------------------------------------------
procedure _SaveTIFF(Bitmap:TXBitmap;FileName:String;ModeA,ModeB:Integer);
{$ifdef USE_GFXLIB_TIFF}
   var aTIFF:TTIFFFile;
{$endif}
begin
{$ifdef USE_GFXLIB_TIFF}
   aTIFF:=TTIFFFile.Create;
   aTIFF.AssignBitmap(Bitmap);
   aTIFF.SaveToFile(FIleName);
   aTIFF.Free;
{$endif}
end;

//---------------------------------------------------------------------------
//
//			_SavePCX
//
//---------------------------------------------------------------------------
procedure _SavePCX(Bitmap:TXBitmap;FileName:String;ModeA,ModeB:Integer);
{$ifdef USE_GFXLIB_PCX}
    var aPcx:TPCXFile;
{$endif}
begin
{$ifdef USE_GFXLIB_PCX}
   aPcx:=TPCXFile.Create;
   aPCX.AssignBitmap(Bitmap);
   aPcx.SaveToFile(FIleName);
   aPcx.Free;
{$endif}
end;

//---------------------------------------------------------------------------
//
//			_SavePNG
//
//---------------------------------------------------------------------------
procedure _SavePNG(Bitmap:TXBitmap;FileName:String;ModeA,ModeB:Integer);
{$ifdef USE_GFXLIB_PNG}
  var aPNG:TPNGImage;
{$endif}
begin
{$ifdef USE_GFXLIB_PNG}
   aPNG:=TPNGImage.Create;
   aPNG.Assign(Bitmap);
   aPNG.SaveToFile(FIleName);
   aPNG.Free;
{$endif}
end;

//---------------------------------------------------------------------------
//
//			_SaveTGA
//
//---------------------------------------------------------------------------
procedure _SaveTGA(Bitmap:TXBitmap;FileName:String;ModeA,ModeB:Integer);
{$ifdef USE_GFXLIB_TGA}
   var aTGA:TTGAFile;
{$endif}
begin
{$ifdef USE_GFXLIB_TGA}
   aTGA:=TTGAFile.Create;
   aTGA.AssignBitmap(Bitmap);
   aTGA.SaveToFile(FIleName);
   aTGA.Free;
{$endif}
end;

//---------------------------------------------------------------------------
//
//			GetProcessedWidth
//
//---------------------------------------------------------------------------
function GetProcessedWidth:Integer;
begin
  GetProcessedWidth:=_ProcessedImgWidth;
end;

//---------------------------------------------------------------------------
//
//			_GetProcessedHeight
//
//---------------------------------------------------------------------------
function GetProcessedHeight:Integer;
begin
  GetProcessedHeight:=_ProcessedImgHeight;
end;

//---------------------------------------------------------------------------
//
//			GetProcessedCRC
//
//---------------------------------------------------------------------------
function GetProcessedCRC:Integer;
begin
  GetProcessedCRC:=_ProcessedCRC32;
end;

//---------------------------------------------------------------------------
//
//			GetProcessedDPI
//
//---------------------------------------------------------------------------
function GetProcessedDPI:Integer;
begin
  GetProcessedDPI:=_ProcessedImgDPI;
end;

//---------------------------------------------------------------------------
//
//			GetProcessedPlanes
//
//---------------------------------------------------------------------------
function GetProcessedPlanes:Integer;
begin
  GetProcessedPlanes:=_ProcessedImgPlanes;
end;

//---------------------------------------------------------------------------
//
//			GetGenerateCRCState
//
//---------------------------------------------------------------------------
function GetGenerateCRCState:Boolean;
begin
  GetGenerateCRCState:=_GenerateCheckSums;
end;

//---------------------------------------------------------------------------
//
//			SetGenerateCRCState
//
//---------------------------------------------------------------------------
function SetGenerateCRCState(State:Boolean):Boolean;
begin
  _GenerateCheckSums:=State;
  SetGenerateCRCState:=_GenerateCheckSums;
end;


//---------------------------------------------------------------------------
//
//			PrintBitmap
//
//---------------------------------------------------------------------------
PROCEDURE PrintBitmap(Canvas:  TxCanvas; DestRect:  TxRect;  Bitmap:  TXBitmap);
VAR
      BitmapHeader:  pxBitmapInfo;
      BitmapImage :  POINTER;
      HeaderSize  :  xDWORD;
      ImageSize   :  xDWORD;
BEGIN
    xGetDIBSizes(Bitmap.Handle, HeaderSize, ImageSize);
    GetMem(BitmapHeader, HeaderSize);
    GetMem(BitmapImage,  ImageSize);
    TRY
      xGetDIB(Bitmap.Handle, Bitmap.Palette, BitmapHeader^, BitmapImage^);
      xStretchDIBits(Canvas.Handle,
                    DestRect.Left, DestRect.Top,     // Destination Origin
                    DestRect.Right  - DestRect.Left, // Destination Width
                    DestRect.Bottom - DestRect.Top,  // Destination Height
                    0, 0,                            // Source Origin
                    Bitmap.Width, Bitmap.Height,     // Source Width & Height
                    BitmapImage,
                    TxBitmapInfo(BitmapHeader^),
                    xDIB_RGB_COLORS,
                    xSRCCOPY)
    FINALLY
      FreeMem(BitmapHeader);
      FreeMem(BitmapImage)
    END
END;

initialization
  SetJPEGDefaults;
  _GenerateCheckSums:=False;
  _ProcessedCRC32:=0;

end.



















