{***********************************************************************
Unit gfx_masks.PAS v1.5 0801
    (c) by Andreas Moser, AMoser@amoser.de

    Delphi version : Delphi 4, 6 (due to user responses it seem to work with Delphi 5)

    gfx_mask is part of the gfx_library collection

    You may use this sourcecode for your freewareproducts.
    You may modify this source-code for your own use.
    You may recompile this source-code for your own use.

    All functions, procedures and classes may NOT be used in commercial
    products without the permission of the author. For parts of this library
    not written by me, you have to ask for permission by their
    respective authors.

    Disclaimer of warranty: "This software is supplied as is. The author
    disclaims all warranties, expressed or implied, including, without
    limitation, the warranties of merchantability and of fitness for any
    purpose. The author assumes no liability for damages, direct or
    consequential, which may result from the use of this software."

    All brand and product names are marks or registered marks of their
    respective companies.

    Please report bugs to:
    Andreas Moser  amoser@amoser.de

    NOTE: This source is adapted from
          "Magic Wand", Francesco Savastano 2001
          Web site : http://digilander.iol.it/gensavas/francosava/

********************************************************************************}

unit gfx_masks;

interface

uses gfx_basedef,gfx_syswrapper;

procedure MaskMagicWand(var SrcBitmap:TXBitmap;a,b:Integer;OldColor:TXColor;Tolerance:Real;var mask:TBool2dArray);
procedure ShowMagicWandFrame(SrcBitmap, DestBitmap:TXBitmap;var mask:TBool2dArray);
procedure ShowMagicWandMask(SrcBitmap,DestBitmap:TXBitmap;var mask:TBool2dArray);
procedure GetMagicWandSelection(SrcBitmap, DestBitmap:TXBitmap;var Mask:TBool2dArray);

implementation

var
  fillx:array[0..80000]of integer;
  filly:array[0..80000]of integer;


  {magic wand algorithm same as seed fill algorithm}
procedure MaskMagicwand(var SrcBitmap:TXBitmap;a,b:Integer;OldColor:TxColor;Tolerance:Real;var Mask:TBool2dArray);
var
  nf:integer;
  i,ir:integer;
  xf,yf:integer;
  BMPIntfc:TBMPInterface;
  crgb:TxRGBTriple;

  procedure MagicWandProcessor(xf,yf:integer;var nf:integer;oldcolor:txcolor;tolerance:real;var mask:tbool2darray);
  var
   pixr,pixg,pixb,oldr,oldg,oldb:byte;
   pix:TxColorref;
   jj:byte;
   xr,yr:longint;
   pp:pRGBarray;
   begin
     xr:=0;yr:=0;
     crgb:=ColorToRGB(OldColor);
     oldr:=crgb.rgbtRed;
     oldg:=crgb.rgbtGreen;
     oldb:=crgb.rgbtBlue;

     for jj:=1 to 4 do
     begin
       case jj of
       1:begin
           xr:=xf+1;
           yr:=yf;
         end;
       2:begin
           xr:=xf-1;
           yr:=yf;
          end;
       3:begin
           xr:=xf;
           yr:=yf+1;
         end;
       4:begin
           xr:=xf;
           yr:=yf-1;
         end;
       end;
       if ((xr<  BMPIntfc.width)and(jj=1))or((xr>=0)and(jj=2)) or ((yr< BMPIntfc.height)and(jj=3))or((yr>=0)and(jj=4)) then
       begin
         pp:=BMPIntfc.SourceScanline[yr];
         pix:=xrgb(pp[xr].rgbtred,pp[xr].rgbtgreen,pp[xr].rgbtblue);
         crgb:=ColorToRGB(pix);
         pixr:=crgb.rgbtRed;
         pixg:=crgb.rgbtGreen;
         pixb:=crgb.rgbtBlue;
         if (not mask[xr,yr])and(abs(pixr-oldr)<=tolerance*150)and(abs(pixg-oldg)<=tolerance*150)and(abs(pixb-oldb)<=tolerance*150)then
         begin
           mask[xr,yr]:=true;
           nf := nf+1;
           fillx[nf]:= xr;
           filly[nf]:= yr;
         end;
       end;
      end;
    end;

begin
  BMPIntfc:=TBMPInterface.Create(SrcBitmap);
  try
  Setlength(Mask,BMPIntfc.width+5,BMPIntfc.height+5);
  nf := 1;
  ir := 1;
  fillx[nf]:= a;
  filly[nf]:= b;
  MagicWandProcessor(a,b,nf,OldColor,Tolerance,Mask);
  while nf>ir do
  begin
    ir := ir+1;
    xf := fillx[ir];
    yf := filly[ir];
    MagicWandProcessor(xf,yf,nf,OldColor,Tolerance,Mask);
    if (nf>75000)  then
    begin
      for i := 1 to nf-ir do
      begin
        fillx[i] := fillx[ir+i];
        filly[i] := filly[ir+i];
      end;
      nf := nf-ir ;
      ir := 0;
    end;
  end;
  finally
    BMPIntfc.Free;
  end;
end;


procedure ShowMagicWandFrame(SrcBitmap,DestBitmap:TXBitmap;var mask:TBool2dArray);
var
  i,j:integer;
  test:boolean;
  pp,pd:pRGBarray;
  BMPIntfc : TBMPInterface1;
begin
  BMPIntfc:=TBMPInterface1.Create(SrcBitmap,DestBitmap,SrcBitmap.PixelFormat,0,True);
  try
  for j:=1 to BMPIntfc.height-2 do
  begin
    pp:=BMPIntfc.SourceScanline[j];
    pd:=BMPIntfc.DestScanline[j];
    for i:=1 to BMPIntfc.width-2 do
    begin
      if mask[i,j]then
      begin
        test:= mask[i-1,j] and mask[i+1,j] and mask[i,j-1] and mask[i,j+1] ;
        if not test then
        begin
          {simply invert the colors of the boundary pixels }
          with pd[i] do
          begin
            rgbtred:=255-pp[i].rgbtred;
            rgbtgreen:=255-pp[i].rgbtgreen;
            rgbtblue:=255-pp[i].rgbtblue;
          end;
        end else
        begin
          with pd[i] do
          begin
            rgbtred:=pp[i].rgbtred;
            rgbtgreen:=pp[i].rgbtgreen;
            rgbtblue:=pp[i].rgbtblue;
          end;
         end;
       end else
        begin
          with pd[i] do
          begin
            rgbtred:=pp[i].rgbtred;
            rgbtgreen:=pp[i].rgbtgreen;
            rgbtblue:=pp[i].rgbtblue;
          end;
        end;

    end;
  end;
  finally
    BMPIntfc.Free;
  end;
end;

procedure ShowMagicWandMask(SrcBitmap,DestBitmap:TXBitmap;var mask:TBool2dArray);
var
  i,j:integer;
  pp,pd:pRGBarray;
  BmpIntfc: TBMPInterface1;
begin
  BmpIntfc:=TBMPInterface1.Create(SrcBitmap,DestBitmap,xpf24bit,0,True);
  try
  for j:=1 to BmpIntfc.height-2 do
  begin
    pp:=BmpIntfc.SourceScanline[j];
    pd:=BmpIntfc.DestScanline[j];
    for i:=1 to BmpIntfc.width-2 do
    begin
      if mask[i,j] then pd[i]:=pp[i];
    end;
  end;
  finally
    BmpIntfc.Free;
  end;
end;

procedure GetMagicWandSelection(SrcBitmap, DestBitmap:TXBitmap;var Mask:TBool2dArray);
var i,j,ul,ut,lr,lb: Integer;
    pp,pd:pRGBarray;
    BmpIntfc: TBMPInterface1;
begin
  BmpIntfc:=TBMPInterface1.Create(SrcBitmap,DestBitmap,xpf24bit,0,False);
  try
    ul:=BmpIntfc.Width;
    ut:=BmpIntfc.Height;
    lr:=0;
    lb:=0;

    //get bounds
    for j:=1 to SrcBitmap.Height-2 do
    begin
      for i:=1 to SrcBitmap.Width-2 do
      begin
         if Mask[i,j] then
         begin
            if j<ut then ut:=j;
            if j>lb then lb:=j;
            if i<ul then ul:=i;
            if i>lr then lr:=i;
         end;
      end;
    end;
    DestBitmap.PixelFormat:=xpf24Bit;
    DestBitmap.Height:= lb-ut;
    DestBitmap.Width:=lr-ul;

    for j:=ut to lb-1 do
    begin
      pp:=BmpIntfc.SourceScanline[j];
      pd:=BmpIntfc.DestScanline[j-ut];
      for i:=ul to lr-1 do
      begin
        if mask[i,j] then pd[i-ul]:=pp[i] else pd[i-ul]:=ColorToRGB(xclNone);
    end;
  end;
 finally
   BmpIntfc.Free;
 end;

end;

end.
