unit gfx_syswrapper;
{***********************************************************************
Unit gfx_syswrapper.PAS v1.5 0801
    (c) by Andreas Moser, amoser@amoser.de,

    Delphi version : Delphi 4, 6 (due to user responses it seem to work with Delphi 5)

    gfx_syswrapper is part of the gfx_library collection

    You may use this sourcecode for your freewareproducts.
    You may modify this source-code for your own use.
    You may recompile this source-code for your own use.

    All functions, procedures and classes may NOT be used in commercial
    products without the permission of the author. For parts of this library
    not written by me, you have to ask for permission by their
    respective authors.

    Disclaimer of warranty: "This software is supplied as is. The author
    disclaims all warranties, expressed or implied, including, without
    limitation, the warranties of merchantability and of fitness for any
    purpose. The author assumes no liability for damages, direct or
    consequential, which may result from the use of this software."

    All brand and product names are marks or registered marks of their
    respective companies.

    Please report bugs to:
    Andreas Moser  amoser@amoser.de

********************************************************************************}

interface
uses windows,graphics,sysutils,messages, classes;

const
    xDIB_RGB_COLORS    = DIB_RGB_COLORS;
    xSRCCOPY           = SRCCOPY;
    xpfDevice          = pfDevice;
    xpf1bit            = pf1bit;
    xpf4bit            = pf4bit;
    xpf8bit            = pf8bit;
    xpf15bit           = pf15bit;
    xpf16bit           = pf16bit;
    xpf24bit           = pf24bit;
    xpf32bit           = pf32bit;
    xPLANES            = PLANES;
    xBITSPIXEL         = BITSPIXEL;
    xRASTERCAPS        = RASTERCAPS;
    xRC_PALETTE        = RC_PALETTE;
    xclWhite           = clWhite;                 {TColor($FFFFFF);}
    xclNone            = clNone;                  {TColor($1FFFFFFF);}
    xPC_NOCOLLAPSE     = PC_NOCOLLAPSE;
    xDEFAULT_PALETTE   = DEFAULT_PALETTE;
    xcmSrcCopy         = cmSrcCopy;               {SRCCOPY}
    xBI_RGB            = BI_RGB;
    xclBlack           = clBlack;                 {TColor($000000);}
    xpsSolid           = psSolid;
    xbsClear           = bsClear;
    xGMEM_MOVEABLE     = GMEM_MOVEABLE;

    type

    TXHandle                     = THandle;
    TXGraphic                    = TGraphic;
    TXBitmap                     = TBitmap;
    TXPicture                    = TPicture;
    TXDIBSection                 = TDIBSection;
    TXMetaFile                   = TMetaFile;
    TXCanvas                     = TCanvas;

    TXPixelFormat                = TPixelFormat;

    TXColor                      = TColor;
    TXColorref                   = TColorref;

    TXBitmapInfo                 = TBitmapInfo;
    TxBitmapInfoHeader           = TBitmapInfoHeader;
    HxDC                         = HDC;
    TxRGBTriple                  = TRGBTriple;
    TXMAXLogPalette              = TMAxLogPalette;
    TXWMQueryNewPalette          = TWMQueryNewPalette;
    TXWMPaletteChanged           = TWMPaletteChanged;
    HxPalette                    = HPalette;
    TXPaletteEntry               = TPaletteEntry;
    pXLogPalette                 = pLogPalette;
    TxLogpalette                 = TLogpalette;
    pXBitmapInfo                 = pBitmapInfo;
    TxRGBQuad                    = TRGBQuad;
    TxBitmapFileHeader           = TBitmapFileHeader;

    TXRect                       = TRect;
    xDWORD                       = DWORD;
    TxProgressEvent              = TProgressEvent;
    pxBytearray                  = pBytearray;
    ExInvalidGraphicOperation    = EInvalidGraphicOperation;
    TxSystemInfo                 = TSystemInfo;
    TxMemoryStream               =  TMemoryStream;


    {some function mappers..so it would be easier to transform gfx_lib to another platform}

    function xIntToStr(Value:Integer):string;
    function xStrToInt(Value:String):Integer;

    procedure xGetDIBSizes(Bitmap: HBITMAP; var InfoHeaderSize: Cardinal; var ImageSize: Cardinal);
    function  xGetDIB(Bitmap: HBITMAP; Palette: HPALETTE; var BitmapInfo; var Bits): Boolean;
    function  xGetDIBits(DC:HDC;Bitmap:HBITMAP;StartScan,NumScans:Cardinal;Bits:Pointer; var BitInfo:TBITMAPINFO;Usage:Cardinal):Integer;
    function  xStretchDIBits(hdc:HDC;XDest,YDest,nDestWidth,nDestHeight,XSrc,YSrc,nSrcWidth,nSrcHeight:Integer;lpBits:pointer;var lpBitsInfo:TBITMAPINFO;iUsage,dwRop:Cardinal):Integer;

    function  xGetPaletteEntries(hPal:HPALETTE;iStartIndex,nEntries:Cardinal; var lppe):Cardinal;
    function  xCreatePalette(const lPal:TLogPalette):HPalette;
    function  xCreateHalfTonePalette(DC:HDC):HPalette;
    function  xGetSystemPaletteEntries(dc:HDC;iStartIndex,nEntries:Cardinal; var lppe):Cardinal;
    function  xCopyPalette(Category: HPALETTE): HPALETTE;
    function  xSelectPalette(DC:HDC;Palette:HPALETTE;ForceBackground:LongBool):HPalette;
    function  xRealizePalette(DC:HDC):Cardinal;

    function  xGetDC(hWnd:HWND):HDC;
    function  xGetDeviceCaps(dc:HDC;Index:Integer):Integer;
    function  xReleaseDC(hWnd:HWND;dc:HDC):Integer;
    function  xUpdateColors(DC:HDC):Boolean;

    function  xGetStockObject(Obj:Integer):HGDIOBJ;
    function  xGetObject(Obj:HGDIOBJ;Buffer:Integer;ObjType:Pointer):Integer;
    function  xDeleteObject(p1:HGDIOBJ):Boolean;

    function  xGDIFlush:Boolean;
    procedure xGetSystemInfo(var SysInfo:TSystemInfo);
    function  xGlobalAllocPtr(Flags,Bytes:Integer):Pointer;
    procedure xGlobalFreePtr(Ptr:Pointer);
    procedure xZeroMemory(Dest:Pointer;Length:Cardinal);

    function  xRect(ALeft, ATop, ARight, ABottom: Integer): TxRect;
    function  xRGB(r,g,b:Byte):TColorRef;
    function  xFileExists(const FileName: string): Boolean;
    function  xExtractFileExt(Filename:String):String;

implementation

// -----------------------------------------------------------------------------
//
//			Function and Procedure Mappers
//
// -----------------------------------------------------------------------------
function xIntToStr(Value:Integer):string;
begin
  result:=IntToStr(Value);
end;

function xStrToInt(Value:String):Integer;
begin
  result:=StrToInt(Value);
end;

procedure xGetDIBSizes(Bitmap: HBITMAP; var InfoHeaderSize: Cardinal; var ImageSize: Cardinal);
begin
     GetDIBSizes(Bitmap,InfoHeaderSize,ImageSize);
end;

function  xGetDIB(Bitmap: HBITMAP; Palette: HPALETTE; var BitmapInfo; var Bits): Boolean;
begin
     result:=GetDIB(Bitmap,palette,bitmapinfo,bits);
end;

function  xStretchDIBits(hdc:HDC;XDest,YDest,nDestWidth,nDestHeight,XSrc,YSrc,nSrcWidth,nSrcHeight:Integer;lpBits:pointer;var lpBitsInfo:TBITMAPINFO;iUsage,dwRop:Cardinal):Integer;
begin
  result:= StretchDIBits(hdc,xdest,ydest,nDestWidth,nDestHeight,xsrc,ysrc,nSrcWidth,nSrcHeight,lpBits,lpBitsInfo,iUsage,dwRop);
end;

function  xGetDC(hWnd:HWND):HDC;
begin
  result:=GetDC(hWnd);
end;

function  xGetDeviceCaps(dc:HDC;Index:Integer):Integer;
begin
  result:=GetDeviceCaps(dc,Index);
end;

function  xReleaseDC(hWnd:HWND;dc:HDC):Integer;
begin
   result:=ReleaseDC(hWnd,dc);
end;

function xCopyPalette(Category: HPALETTE): HPALETTE;
begin
  Result:=CopyPalette(Category);
end;

function xDeleteObject(p1:HGDIOBJ):Boolean;
begin
  result:=DeleteObject(p1);
end;

function  xRect(ALeft, ATop, ARight, ABottom: Integer): TxRect;
begin
  Result:=Rect(ALeft,Atop,ARight,ABottom);
end;

function  xGetPaletteEntries(hPal:HPALETTE;iStartIndex,nEntries:Cardinal; var lppe):Cardinal;
begin
  Result:=GetPaletteEntries(hPal,iStartIndex,nEntries,lppe);
end;

function  xCreatePalette(const lPal:TLogPalette):HPalette;
begin
  Result:=CreatePalette(lPal);
end;

function  xCreateHalfTonePalette(DC:HDC):HPalette;
begin
   Result:=CreateHalftonePalette(DC);
end;

function xGetSystemPaletteEntries(dc:HDC;iStartIndex,nEntries:Cardinal; var lppe):Cardinal;
begin
  Result:=xGetSystemPaletteEntries(dc,iStartIndex,nEntries,lppe);
end;

function xGetStockObject(Obj:Integer):HGDIOBJ;
begin
  Result:=GetStockObject(Obj);
end;

function xSelectPalette(DC:HDC;Palette:HPALETTE;ForceBackground:LongBool):HPalette;
begin
    result:=SelectPalette(DC,Palette,ForceBackground);
end;

function xRealizePalette(DC:HDC):Cardinal;
begin
  Result:=RealizePalette(DC);
end;

function xUpdateColors(DC:HDC):Boolean;
begin
  Result:=UpdateColors(DC);
end;

function xRGB(r,g,b:Byte):TColorRef;
begin
  Result:=rgb(r,g,b);
end;

procedure xZeroMemory(Dest:Pointer;Length:Cardinal);
begin
  ZeroMemory(Dest,Length);
end;

function xGetDIBits(DC:HDC;Bitmap:HBITMAP;StartScan,NumScans:Cardinal;Bits:Pointer; var BitInfo:TBITMAPINFO;Usage:Cardinal):Integer;
begin
  Result:=GetDIBits(dc,Bitmap,StartScan,NumScans,Bits,BitInfo,Usage);
end;

function  xGetObject(Obj:HGDIOBJ;Buffer:Integer;ObjType:Pointer):Integer;
begin
  Result:=GetObject(Obj,Buffer,ObjType);
end;

function xFileExists(const FileName: string): Boolean;
begin
  Result:= FileExists(FileName);
end;

function  xExtractFileExt(Filename:String):String;
begin
  result:= ExtractFileExt(FileName);
end;

function xGDIFlush:Boolean;
begin
 result:=GDIFlush;
end;

procedure xGetSystemInfo(var SysInfo:TSystemInfo);
begin
  GetSystemInfo(SysInfo);
end;

function xGlobalAllocPtr(Flags,Bytes:Integer):Pointer;
begin
 result := GlobalAllocPtr(Flags,Bytes);
end;

procedure xGlobalFreePtr(Ptr:Pointer);
begin
 GlobalFreePtr(Ptr);
end;

end.
